<?php

/*
 * OpenCart
 * Модуль для подключения платежной системы Z-Payment
 *
 */
?>
<?php

class ControllerPaymentZpayment extends Controller {

    function ob_exit($status = null) {
        if ($status) {
            ob_end_flush();
            isset($_REQUEST['debug']) ? exit($status) : exit();
        } else {
            ob_end_clean();
            header("HTTP/1.0 200 OK");
            echo "OK";
            exit();
        }
    }

    function debug_file() {
        header('Content-type: text/plain; charset=utf-8');
        echo file_get_contents(__FILE__);
    }

    function check_im() {
        $iplist = gethostbynamel('z-payment.com');
        foreach ($iplist as $ip)
            if (preg_match("/^$ip/", (isset($_SERVER['HTTP_REFERER'])) ? $_SERVER['HTTP_REFERER'] : ''))
                return true;
        return false;
    }

    function from_request($name) {
        return isset($this->request->request[$name]) ? html_entity_decode($this->request->request[$name]) : null;
    }

    public function index() {
        $data['button_confirm'] = $this->language->get('button_confirm');
        $data['button_back'] = $this->language->get('button_back');


        $data['action'] = 'https://z-payment.com/merchant.php';

        $this->load->model('checkout/order');

        $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

        $products = '';

        foreach ($this->cart->getProducts() as $product) {
            $products .= $product['quantity'] . ' x ' . $product['name'] . ', ';
        }
        
        $amount = number_format($order_info['total'], 2, '.', '');
        $idShop = $this->config->get('zpayment_eshopid');
        $idOrder = $this->session->data['order_id'];
        $desc = "Payment ".$idOrder ." from shop ".$idShop;
        $initPass = $this->config->get('zpayment_initialization_password'); 
        $ZP_SIGN = md5($idShop.$idOrder.$amount.$initPass);
        $data['LMI_PAYEE_PURSE'] = $idShop;
        $data['CLIENT_MAIL'] = $order_info['email'];
        $data['LMI_PAYMENT_AMOUNT'] = $amount;
        $data['LMI_PAYMENT_DESC'] = $desc;
        $data['LMI_PAYMENT_NO'] = $idOrder;
        $data['ZP_SIGN'] = $ZP_SIGN;
        if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/zpayment.tpl')) {
            return $this->load->view($this->config->get('config_template') . '/template/payment/zpayment.tpl', $data);
        } else {
            return $this->load->view('default/template/payment/zpayment.tpl', $data);
        }
    }

    public function callback() {
        ob_start();

        if (isset($_REQUEST['LMI_PREREQUEST']) && $_REQUEST['LMI_PREREQUEST'] ==1) {
            echo 'YES';
            die();
        }
        if (!isset($_REQUEST['LMI_PAYMENT_NO'])) {
            $this->ob_exit("ERROR: EMPTY REQUEST!\n");
        }

        $LMI_PAYMENT_NO = $this->from_request('LMI_PAYMENT_NO');
        $LMI_PAYEE_PURSE = $this->from_request('LMI_PAYEE_PURSE');
        $LMI_PAYMENT_AMOUNT = $this->from_request('LMI_PAYMENT_AMOUNT');
        $LMI_MODE = $this->from_request('LMI_MODE');
        $LMI_SYS_INVS_NO = $this->from_request('LMI_SYS_INVS_NO');
        $LMI_SYS_TRANS_NO = $this->from_request('LMI_SYS_TRANS_NO');
        $LMI_SYS_TRANS_DATE = $this->from_request('LMI_SYS_TRANS_DATE');
        $LMI_PAYER_PURSE = $this->from_request('LMI_PAYER_PURSE');
        $LMI_PAYER_WM = $this->from_request('LMI_PAYER_WM');
        $LMI_HASH = $this->from_request('LMI_HASH');
        $LMI_SECRET_KEY = $this->from_request('LMI_SECRET_KEY');

        $shopId = $this->config->get('zpayment_eshopid');
        $shopSecret = $this->config->get('zpayment_secret_key');
        $err = FALSE;
 
        $this->load->model('checkout/order');
        $order_info = $this->model_checkout_order->getOrder($LMI_PAYMENT_NO);

        // ORDER ID CHECK
        if (!$order_info) {
            $this->ob_exit("ERROR: ORDER NOT EXISTS!\n");
        }
        $amount = number_format($order_info['total'], 2, '.', '');
        $this->model_checkout_order->addOrderHistory($LMI_PAYMENT_NO, $this->config->get('config_order_status_id'));
        if ($amount != $LMI_PAYMENT_AMOUNT) {
             $err=TRUE;
             $errText="Amount mismatch";
             $this->ob_exit($errText);
        }
        if ($shopId!=$LMI_PAYEE_PURSE) {
             $err=TRUE;
             $errText="Shop Id mismatch";
             $this->ob_exit($errText);
        }
        if ($LMI_SECRET_KEY) {
           if($LMI_SECRET_KEY!=$shopSecret) {
               $err = TRUE;
               $errText='secret key mismatch';
               $this->ob_exit($errText);   
            }
        } else {
            $CalcHash = md5($LMI_PAYEE_PURSE.$LMI_PAYMENT_AMOUNT.$LMI_PAYMENT_NO.$LMI_MODE.$LMI_SYS_INVS_NO.$LMI_SYS_TRANS_NO.$LMI_SYS_TRANS_DATE.$shopSecret.$LMI_PAYER_PURSE.$LMI_PAYER_WM);
           if($LMI_HASH != strtoupper($CalcHash)) { 
               $err = TRUE;
               $errText='hash mismatch';
               $this->ob_exit($errText);
           }
        }
        if (!$err) {
            $this->model_checkout_order->addOrderHistory($LMI_PAYMENT_NO, $this->config->get('zpayment_order_status_id'), 'ZPayment', False);
            $this->ob_exit();
            echo 'YES';
            die();
        }
    }

}

?>
